SUPPORTED_LEAGUES = {
    "PL": {
        "icon": "🏴󠁧󠁢󠁥󠁮󠁧󠁿",
        "name": "Premier League",
    },
    "CL": {
        "icon": "🏆",
        "name": "Champions League",
    },
    # "WC": {
    #     "icon": "🌍",
    #     "name": "World Cup",
    # },
    "PD": {
        "icon": "🇪🇸 ",
        "name": "La Liga",
    },
    "SA": {
        "icon": "🇮🇹 ",
        "name": "Serie A",
    },
    "BL1": {
        "icon": "🇩🇪 ",
        "name": "Bundesliga",
    },
    "FL1": {
        "icon": "🇫🇷 ",
        "name": "Ligue 1",
    },
}

DEFAULT_LEAGUE = "PL"
